#include "xipipsu.h"

XIpiPsu_Config XIpiPsu_ConfigTable[] __attribute__ ((section (".drvcfg_sec"))) = {

	{
		"xlnx,versal-ipi-mailbox", /* compatible */
		0xeb350000, /* reg */
		0x10, /* xlnx,ipi-bitmask */
		0x4, /* xlnx,ipi-buf-index */
		0x403b, /* interrupts */
		0xe2000000, /* interrupt-parent */
		0x10, /* xlnx,ipi-target-count */
		{
			{
				1, /* xlnx,ipi-bitmask */
				0 /* xlnx,ipi-buf-index */
			},
			{
				4, /* xlnx,ipi-bitmask */
				2 /* xlnx,ipi-buf-index */
			},
			{
				8, /* xlnx,ipi-bitmask */
				3 /* xlnx,ipi-buf-index */
			},
			{
				1024, /* xlnx,ipi-bitmask */
				65535 /* xlnx,ipi-buf-index */
			},
			{
				16, /* xlnx,ipi-bitmask */
				4 /* xlnx,ipi-buf-index */
			},
			{
				2048, /* xlnx,ipi-bitmask */
				65535 /* xlnx,ipi-buf-index */
			},
			{
				32, /* xlnx,ipi-bitmask */
				5 /* xlnx,ipi-buf-index */
			},
			{
				4096, /* xlnx,ipi-bitmask */
				65535 /* xlnx,ipi-buf-index */
			},
			{
				64, /* xlnx,ipi-bitmask */
				6 /* xlnx,ipi-buf-index */
			},
			{
				8192, /* xlnx,ipi-bitmask */
				65535 /* xlnx,ipi-buf-index */
			},
			{
				128, /* xlnx,ipi-bitmask */
				7 /* xlnx,ipi-buf-index */
			},
			{
				16384, /* xlnx,ipi-bitmask */
				65535 /* xlnx,ipi-buf-index */
			},
			{
				512, /* xlnx,ipi-bitmask */
				65535 /* xlnx,ipi-buf-index */
			},
			{
				32768, /* xlnx,ipi-bitmask */
				65535 /* xlnx,ipi-buf-index */
			},
			{
				2, /* xlnx,ipi-bitmask */
				1 /* xlnx,ipi-buf-index */
			},
			{
				256, /* xlnx,ipi-bitmask */
				65535 /* xlnx,ipi-buf-index */
			},
		}, /* child,required */
	},
	 {
		 NULL
	}
};